@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -22,7 +22,9 @@
 
 ===============================================================================
 
-See django.core.management.commands.startproject.Command.handle
+**Generates a new random secret key suitable for use with Django.**
+
+See ``django.core.management.commands.startproject.Command.handle``.
 """
 
 from django.utils.crypto import get_random_string
