@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -22,8 +22,8 @@
 
 ===============================================================================
 
-Drug information, with an emphasis on psychotropic drugs, including translating
-specific to generic names.
+**Drug information, with an emphasis on psychotropic drugs, including 
+translating specific to generic names.**
 
 **Examples**
 
@@ -202,11 +202,19 @@ from typing import Dict, List, Optional, Union
 WILDCARD = ".*"  # if re.DOTALL is set, this also matches newlines
 WB = WORD_BOUNDARY = r"\b"
 
+
 # =============================================================================
 # Class to capture drug information
 # =============================================================================
 
 class Drug(object):
+    """
+    Class to describe a specific drug, or a drug category.
+
+    Also embodies knowledge about brand names and common misspellings.
+
+    See the :const:`DRUGS` list for example of use.
+    """
     def __init__(
             self,
             # Names
@@ -261,15 +269,93 @@ class Drug(object):
             vitamin: bool = False,
             # Special flags:
             slam_antidepressant_finder: bool = False) -> None:
+        # noinspection PyUnresolvedReferences
         """
         Initialize and determine/store category knowledge.
 
-        "alternatives" can include regexes (as text).
+        ``alternatives`` can include regexes (as text).
 
         We add front/back wildcards by default; this handles all situations
         like "depot X", etc. We also add a preceding word boundary (after the
         wildcard); thus the usual transformation is ``XXX`` -> ``.*\bXXX.*``.
 
+        Args:
+            generic: generic name, or list of names
+            alternatives: can include regexes (as text)
+
+            category_not_drug: is this a drug category, not a specific drug?
+
+            add_preceding_wildcards: when making a regex (etc.), add a wildcard
+                to the start of all possibilities (generic + alternative names)
+                that don't already have one?
+            add_preceding_word_boundary: when making a regex (etc.), add word
+                boundaries to the start of all possibilities (generic +
+                alternative names) that don't already have one?
+            add_following_wildcards: when making a regex (etc.), add a wildcard
+                to the end of all possibilities (generic + alternative names)
+                that don't already have one?
+
+            psychotropic: a psychotropic drug?
+
+            antidepressant: an antidepressant?
+            conventional_antidepressant: a traditional antidepressant?
+            ssri: a selective serotonin reuptake inhibitor (SSRI)?
+            non_ssri_modern_antidepressant: a non-SSRI "modern" antidepressant?
+            tricyclic_antidepressant: a tricyclic?
+            tetracyclic_and_related_antidepressant: a tetracyclic or related?
+            monoamine_oxidase_inhibitor: a MAO-I?
+
+            antipsychotic: an antipsychotic?
+            first_generation_antipsychotic: an FGA?
+            second_generation_antipsychotic: an SGA?
+
+            stimulant: a psychostimulant?
+
+            anticholinergic: an anticholinergic?
+
+            benzodiazepine: a benzodiazepine?
+            z_drug: a "Z" drug (e.g. zopiclone, zolpidem, ...)
+            non_benzodiazepine_anxiolytic: a non-BZ anxiolytic?
+            gaba_a_functional_agonist: a GABA-A functional agonist?
+            gaba_b_functional_agonist: a GABA-B functional agonist?
+
+            mood_stabilizer: a "mood stabilizer"?
+
+            antidiabetic: treats diabetes?
+            sulfonylurea: a sulfonylurea (sulphonylurea), for diabetes?
+            biguanide: a biguanide, for diabetes?
+            glifozin: a glifozin, for diabetes?
+            glp1_agonist: a GLP-1 agonist, for diabetes?
+            dpp4_inhibitor: a DPP4 inhibitor, for diabetes?
+            meglitinide: a meglitinide, for diabetes?
+            thiazolidinedione: a thiazolidinedione, for diabetes?
+
+            cardiovascular: a cardiovascular drug?
+            beta_blocker: a beta adrenoceptor antagonist?
+            ace_inhibitor: an ACE inhibitor?
+            statin: a statin?
+
+            respiratory: a respiratory drug?
+            beta_agonist: a beta adrenoceptor agonist?
+
+            gastrointestinal: a gastrointestinal drug?
+            proton_pump_inhibitor: a PPI?
+            nonsteroidal_anti_inflammatory: an NSAID?
+
+            vitamin: a vitamin?
+
+            slam_antidepressant_finder: a drug found by the SLAM
+                antidepressant-finding code? (A bit specialized, this one!)
+
+        Attributes:
+
+            mixture (bool): is this a mixture of more than one drug?
+                Will be set if more than one generic name is given.
+            all_generics (List[str]): list of all generic names in lower case
+            generic_name: generic name (or combination name like ``a_with_b``
+                for mixtures of ``a`` and ``b``)
+            regex: compiled case-insensitive regular expression to match
+                possible names
         """
         # ---------------------------------------------------------------------
         # Name handling
@@ -389,6 +475,10 @@ class Drug(object):
 
     def name_matches(self, name: str) -> bool:
         """
+        Detects whether the name that's passed matches our knowledge of any of
+        things that this drug might be called: generic name, brand name(s),
+        common misspellings.
+
         The parameter should be pre-stripped of edge whitespace.
         """
         return bool(self.regex.match(name))
@@ -1034,9 +1124,13 @@ def get_drug(drug_name: str,
              name_is_generic: bool = False,
              include_categories: bool = False) -> Optional[Drug]:
     """
-    Converts a drug name to a Drug class.
+    Converts a drug name to a :class:`.Drug` class.
+
     If you already have the generic name, you can get the Drug more
-    efficiently.
+    efficiently by setting ``name_is_generic=True``.
+
+    Set ``include_categories=True`` to include drug categories (such as
+    tricyclics) as well as individual drugs.
     """
     drug_name = drug_name.strip().lower()
     if name_is_generic:
@@ -1060,7 +1154,7 @@ def drug_name_to_generic(drug_name: str,
                          default: str = None,
                          include_categories: bool = False) -> str:
     """
-    Converts a drug name to a generic equivalent.
+    Converts a drug name to the name of its generic equivalent.
     """
     drug = get_drug(drug_name, include_categories=include_categories)
     if drug is not None:
@@ -1073,10 +1167,14 @@ def drug_names_to_generic(drugs: List[str],
                           default: str = None,
                           include_categories: bool = False) -> List[str]:
     """
-    Converts a list of drug names to their generics equivalents.
+    Converts a list of drug names to their generic equivalents.
 
-    From R via reticulate, when using e.g. the ``default`` parameter and
-    storing results in a data.table() character column:
+    The arguments are as for :func:`drug_name_to_generic` but this function
+    handles a list of drug names rather than a single one.
+
+    Note in passing the following conversion of blank-type representations from
+    R via ``reticulate``, when using e.g. the ``default`` parameter and storing
+    results in a ``data.table()`` character column:
 
     .. code-block:: none
 
@@ -1104,6 +1202,16 @@ def drug_names_to_generic(drugs: List[str],
 # =============================================================================
 
 def drug_matches_criteria(drug: Drug, **criteria: Dict[str, bool]) -> bool:
+    """
+    Determines whether a drug, passed as an instance of :class:`.Drug`, matches
+    the specified criteria.
+
+    Args:
+        drug: a :class:`.Drug` instance
+        criteria: ``name=value`` pairs to match against the attributes of
+            the :class:`Drug` class. For example, you can include keyword
+            arguments like ``antidepressant=True``.
+    """
     for attribute, value in criteria.items():
         if getattr(drug, attribute) != value:
             return False
@@ -1114,6 +1222,11 @@ def all_drugs_where(sort=True,
                     include_categories: bool = False,
                     **criteria: Dict[str, bool]) -> List[Drug]:
     """
+    Find all drugs matching the specified criteria (see
+    :func:`drug_matches_criteria`). If ``include_categories`` is true, then
+    drug categories (like "tricyclics") are included as well as individual
+    drugs.
+
     Pass keyword arguments such as
 
     .. code-block:: python
@@ -1139,6 +1252,10 @@ def drug_name_matches_criteria(drug_name: str,
                                name_is_generic: bool = False,
                                include_categories: bool = False,
                                **criteria: Dict[str, bool]) -> bool:
+    """
+    Establish whether a single drug, passed by name, matches the specified
+    criteria. See :func:`drug_matches_criteria`.
+    """
     drug = get_drug(drug_name, name_is_generic)
     if drug is None:
         return False
@@ -1151,6 +1268,10 @@ def drug_names_match_criteria(drug_names: List[str],
                               names_are_generic: bool = False,
                               include_categories: bool = False,
                               **criteria: Dict[str, bool]) -> List[bool]:
+    """
+    Establish whether multiple drugs, passed as a list of drug names, each
+    matches the specified criteria. See :func:`drug_matches_criteria`.
+    """
     return [
         drug_name_matches_criteria(
             dn,
